#!/usr/bin/env roundup

describe libreblacklist
. ./test-common.sh

_blacklist_url=https://projects.parabola.nu/blacklist.git/plain/blacklist.txt

it_works_with_just_pkgname() {
	v="$(libreblacklist normalize <<<skype)"; [[ $v == 'skype::::' ]]
	v="$(libreblacklist get-pkg <<<skype)"; [[ $v == skype ]]
	libreblacklist get-rep <<<irreplaceable | equals $'\n'
	libreblacklist get-url <<<skype | equals $'\n'
	libreblacklist get-reason <<<skype | equals $'\n'
}

it_works_with_everything_set() {
	line='linux:conflict:parabola:id:[semifree] blobs and firmware'
	v="$(libreblacklist normalize <<<"$line")"; [[ $v == "$line" ]]
	v="$(libreblacklist get-pkg <<<"$line")"; [[ $v == 'linux' ]]
	v="$(libreblacklist get-rep <<<"$line")"; [[ $v == 'libre/linux-libre' ]]
	v="$(libreblacklist get-url <<<"$line")"; [[ $v == 'https://labs.parabola.nu/issues/id' ]]
	v="$(libreblacklist get-reason <<<"$line")"; [[ $v == '[semifree] blobs and firmware' ]]
}

it_normalizes_correctly() {
	libreblacklist normalize <<<'#comment' | equals ''
	v="$(libreblacklist normalize <<<pkg)"; [[ $v == 'pkg::::' ]]
	v="$(libreblacklist normalize <<<pkg:)"; [[ $v == 'pkg::::' ]]
	v="$(libreblacklist normalize <<<pkg::)"; [[ $v == 'pkg::::' ]]
	v="$(libreblacklist normalize <<<pkg:rep)"; [[ $v == 'pkg:rep:::' ]]
	v="$(libreblacklist normalize <<<pkg:rep:)"; [[ $v == 'pkg:rep:::' ]]
	v="$(libreblacklist normalize <<<pkg:rep:ref)"; [[ $v == 'pkg:rep:ref::' ]]
	v="$(libreblacklist normalize <<<pkg:rep:ref:)"; [[ $v == 'pkg:rep:ref::' ]]
	v="$(libreblacklist normalize <<<pkg:rep:ref:id)"; [[ $v == 'pkg:rep:ref:id:' ]]
	v="$(libreblacklist normalize <<<pkg:rep:ref:id:)"; [[ $v == 'pkg:rep:ref:id:' ]]
	v="$(libreblacklist normalize <<<pkg:rep:ref:id:reason)"; [[ $v == 'pkg:rep:ref:id:reason' ]]
}

it_works_with_colons_in_reason() {
	line='package:conflict:parabola:id:my:reason'
	v="$(libreblacklist normalize <<<"$line")"; [[ $v == "$line" ]]
	v="$(libreblacklist get-pkg <<<"$line")"; [[ $v == 'package' ]]
	libreblacklist get-rep <<<"$line" | equals $'\n'
	v="$(libreblacklist get-url <<<"$line")"; [[ $v == 'https://labs.parabola.nu/issues/id' ]]
	v="$(libreblacklist get-reason <<<"$line")"; [[ $v == 'my:reason' ]]
}

it_prints_urls_only_for_valid_references() {
	libreblacklist get-url <<<package:::id: | equals $'\n'
	libreblacklist get-url <<<package::unknown:id: | equals $'\n'
}

it_fails_update_with_no_blacklist_or_network() {
	mkdir -p $XDG_CONFIG_HOME/libretools
	echo "BLACKLIST='phony://example.com'" >$XDG_CONFIG_HOME/libretools/libretools.conf

	libreblacklist update >$tmpdir/stdout 2>$tmpdir/stderr || stat=$?

	[[ $stat != 0 ]]
	empty $tmpdir/stdout
	not empty $tmpdir/stderr
}

it_fails_cat_with_no_blacklist_or_network() {
	mkdir -p $XDG_CONFIG_HOME/libretools
	echo "BLACKLIST='phony://example.com'" >$XDG_CONFIG_HOME/libretools/libretools.conf

	libreblacklist cat >$tmpdir/stdout 2>$tmpdir/stderr || stat=$?

	[[ $stat != 0 ]]
	empty $tmpdir/stdout
	not empty $tmpdir/stderr
}

it_fails_update_when_BLACKLIST_is_unset() {
	mkdir -p $XDG_CONFIG_HOME/libretools
	echo "BLACKLIST=" >$XDG_CONFIG_HOME/libretools/libretools.conf

	libreblacklist update >$tmpdir/stdout 2>$tmpdir/stderr || stat=$?

	[[ $stat != 0 ]]
	empty $tmpdir/stdout
	not empty $tmpdir/stderr
}

it_fails_cat_when_syntax_error_in_conf() {
	mkdir -p $XDG_CONFIG_HOME/libretools
	# there is a stray single quote in there
	printf "BLACKLIST='%q\n" "${_blacklist_url}" >$XDG_CONFIG_HOME/libretools/libretools.conf

	libreblacklist cat >$tmpdir/stdout 2>$tmpdir/stderr || stat=$?

	[[ $stat != 0 ]]
	empty $tmpdir/stdout
	not empty $tmpdir/stderr
}

it_downloads_the_blacklist_as_needed() {
	require network || return 0
	mkdir -p $XDG_CONFIG_HOME/libretools
	printf 'BLACKLIST=%q\n' "${_blacklist_url}" >$XDG_CONFIG_HOME/libretools/libretools.conf

	libreblacklist cat >$tmpdir/stdout 2>$tmpdir/stderr

	not empty $tmpdir/stdout
}

it_downloads_the_blacklist_repeatedly() {
	require network || return 0
	mkdir -p $XDG_CONFIG_HOME/libretools
	printf 'BLACKLIST=%q\n' "${_blacklist_url}" >$XDG_CONFIG_HOME/libretools/libretools.conf

	libreblacklist update
	libreblacklist update
}

it_displays_help_and_fails_with_no_args() {
	LC_ALL=C libreblacklist >$tmpdir/stdout 2>$tmpdir/stderr || stat=$?

	[[ $stat != 0 ]]
	empty $tmpdir/stdout
	[[ "$(sed 1q $tmpdir/stderr)" =~ 'Usage: libreblacklist ' ]]
}

it_displays_help_when_given_h() {
	LC_ALL=C libreblacklist -h >$tmpdir/stdout 2>$tmpdir/stderr

	[[ "$(sed 1q $tmpdir/stdout)" =~ 'Usage: libreblacklist ' ]]
	empty $tmpdir/stderr
}

it_displays_help_when_given_h_cat() {
	LC_ALL=C libreblacklist -h cat >$tmpdir/stdout 2>$tmpdir/stderr

	[[ "$(sed 1q $tmpdir/stdout)" == 'Usage: libreblacklist cat' ]]
	empty $tmpdir/stderr
}
